#!/bin/sh
# the next line restarts using tclsh\
exec tclsh "$0" "$@"

# DIMM interface simulator program.
# Port number:
set port 1236
# Identification:
set ident "turbina.tcl v0.02"

set time_of_data ""
set isopl 0
#
#... Return the local time stamp according to the adopted format:
#
set ut_format "%Y-%m-%d %H:%M:%S"
proc time_stamp {{sec1969 -1} {gmt true}} {
	global ut_format
	if {$sec1969==-1} {
		set sec1969 [clock seconds]
	}
	return [clock format $sec1969 -format $ut_format -gmt $gmt]
}

# Sumulator of DIMM data
proc refresh_data {} {
	global isopl time_of_data
	if {$isopl==0} {set isopl 0.5}
	set isopl [expr $isopl + rand()/2-0.25]
	if {$isopl<0.25} {set isopl 0.3} elseif {$isopl>3.0} {set isopl 1.5}
	set time_of_data [time_stamp]
}

# Compose a string with latest obtained results
proc get_data {} {
	global time_of_data isopl
	set s "ut=\\\"$time_of_data\\\" isopl=[format %4.2f $isopl]"
}

proc cmd_get pars {
	# global array of parameter values cmp_pvar(param)=value and comp.status:
	global cmp_pvar cmp_status ident
	foreach p $pars {
		switch $p {
			"ident"
				{ set cmp_pvar($p) $ident }
			"data"
				{ set cmp_pvar($p) [get_data] }
			default
				{ set cmp_status ERSYN; break }
		}
	}
}

proc cmd_init {} {
	global cmp_status cmp_pvar after_id
	# preparation time in seconds:
	set t_init 5
	# read CFG file?
	# simulate hardware preparation:
	set cmp_status BUSY
	set cmp_pvar(wait) $t_init
	set after_id [after [expr $t_init*1000] {
		set cmp_status READY
	}]
}

proc cmd_go_local {} {
	global cmp_status
	# enable accepting local console commands
	set cmp_status LOCAL
}

proc cmd_park {} {
	global cmp_status cmp_pvar after_id
	# simulate hardware shutdown
	set t_park 17
	set cmp_status BUSY
	set cmp_pvar(wait) $t_park
	set after_id [after [expr $t_park*1000] {
		set cmp_status PARKED
	}]
}

# timer identifier of run command: to be able to simulate its stop correctly
set after_id 0
proc cmd_run args {
	global cmp_status cmp_pvar after_id
	puts "args=$args"
	set cmp_status BUSY
	set t_bkgr 5; set t_star 17
	if {[string match -nocase "*bkgr switch*" $args]} {
		puts "Measure background during $t_bkgr sec..."
		set t_job $t_bkgr
	} else {
		puts "Measure isopl during $t_star sec..." 
		set t_job $t_star
		# simulate data:
		refresh_data
	}
	#...
	set cmp_pvar(wait) $t_job
	set after_id [after [expr $t_job*1000]\
	 { puts "done";set cmp_status READY; set after_id 0 }]
}

proc cmd_stop args {
	global after_id cmp_status
	# stop accumulation of data under way.........
	# simulate it:
	if {$after_id!=0} {
		set cmp_status READY
		puts "Long action stopped"
		after cancel $after_id
		set after_id 0
	}
}

# object set in DIMM
set object_hr 0
# setting the parameters from list p1 v1 p2 v2... sent by name for upvar ref.
proc cmd_set pv_listname {
	global cmp_status object_hr
	upvar $pv_listname pv_list
	array set pvar $pv_list
	# - would be enough if pvar is made global, trusting input completely. 
	foreach pname [array names pvar] {
		switch $pname {
			"object"
				{
					set hr $pvar($pname)
					puts -nonewline "Got object=$hr  "
					# simulation of checking object and identifying it in
					# local catalog:
					if {[string is digit $hr] && $hr<9876} {
						# HR ok
						set object_hr $hr
						puts "Set HR=$object_hr"
					} else {
						puts "Bad object ID"
						set cmp_status ERANG
						break
					}
				}
			default
				{
					set cmp_status ERSYN
					break
				}
		}
	}
}

proc cmd_go_online {} {
	global cmp_status
	# wait until local job is completed here:
	#............. vwait ...
	# disable local console commands.........
	set cmp_status READY
}

source compcmd.tcl

open_server $port
#while 1 { after 1000; puts "hsun=[format %-3.1f [compute_hsun]] deg" }
exit
