#!/bin/sh
# the next line restarts using tclsh\
exec tclsh "$0" "$@"

# Meteo service program prototype
# Port number:
set port 1237
# Identification:
set ident "meteo.tcl v0.01"

# global parameters 
# Temperature [centigrees]:
set par_t 10 
# Humidity [%]
set par_h 50 
# Pressure [mbar]
set par_p 700
# Wind speed [m/s]
set par_w 0
# Wind azimuth [N->E, degree]
set par_wd 0
# Rain flag (0/1)
set par_r 0

# Settings of limits of parameters
array set weather_limit { t_min -10 t_max 35 h_min 0 h_max 95 p_min 600 
p_max 2000 w_min 0 w_max 15 wd_min 0 wd_max 360 r_min 0 r_max 0 }

set par_list {t h p w wd r}

# Utility that refreshs the global parameters:
proc request_par {} {
	global par_t par_h par_p par_w par_wd par_r
	# here we model requesting with random values:
	set par_t [expr $par_t + rand()*4-2] 
	if {$par_t<-10} {set par_t -10} elseif {$par_t>25} {set par_t 25}
	
	set par_h [expr $par_h + int(rand()*20-10)]
	if {$par_h<0} {set par_h 0} elseif {$par_h>100} {set par_h 90}

	set par_p [expr $par_p + int(rand()*20-10)]
	if {$par_p<650} {set par_p 650} elseif {$par_p>750} {set par_p 750}

	set par_w [expr $par_w + int(rand()*14-7)]
	if {$par_w<0} {set par_w 0} elseif {$par_w>18} {set par_w 8}

	set par_wd [expr $par_wd + int(rand()*20-10)]
	if {$par_wd<0} {
		set par_wd [expr $par_wd + 360]
	} elseif {$par_wd>360} {set	par_wd [expr $par_wd - 360]}

	set par_r 0
	if {$par_h>80} {
		set t [expr rand()*20]
		if {$t<[expr $par_h-80]} {set par_r 1}
	}

}

# returns the string "param1=value1 param2=value2 ..." of weather conditions.
proc get_data {} {
	global par_list
	# refresh data:
	request_par
	# compose string:
	foreach p $par_list {
		upvar #0 par_$p par
		if {$p=="t"} {
			lappend s $p=[format %-3.1f $par]
		} else { lappend s $p=$par}
	}
	return $s
}

# returns flag of weather condition acceptable for observations:
proc is_weather_good {} {
	global weather_limit par_list
	# refresh data:
	request_par
	set is_good 1
	foreach p $par_list {
		upvar #0 par_$p par
		if {$par<$weather_limit(${p}_min) || $par>$weather_limit(${p}_max)} {
			set is_good 0
			break
		}
	}
	return $is_good
}

proc cmd_get pars {
	# global array of parameter values cmp_pvar(param)=value and comp.status:
	global cmp_pvar cmp_status ident
	foreach p $pars {
		switch $p {
			"ident"
				{ set cmp_pvar($p) $ident }
			"data"
				{ set cmp_pvar($p) [get_data] }
			"cond"
				{ 
					if {[is_weather_good]} {
						set cmp_pvar($p) GOOD
					} else {
						set cmp_pvar($p) BAD
					}
				}
			default
				{ set cmp_status ERSYN; break }
		}
	}
}

proc cmd_init {} {
	global cmp_status
	# read CFG file?
	set cmp_status READY
}

proc cmd_go_local {} {
	global cmp_status
	# enable accepting local console commands
	set cmp_status LOCAL
}

proc cmd_park {} {
	global cmp_status cmp_pvar
	# possibly do nothing?
	# here we model long-term process:
#	set cmp_status BUSY
#	set cmp_pvar(wait) 10
#	after 10000 {}
		set cmp_status PARKED
#	{}
}

proc cmd_go_online {} {
	global cmp_status
	# disable local console commands
	set cmp_status READY
}

source compcmd.tcl

set cmp_status READY
open_server $port
#for {set i 0} {$i<100} {incr i} { puts "[get_data]  is_good=[is_weather_good]"}
exit
