#!/bin/sh
# the next line restarts using tclsh\
exec wish "$0" "$@"

set cmdnum 0
proc cmd {DEST COMMAND {args no}} {
	puts "[clock seconds]: $cmdnum ->$DEST $COMMAND $args"
	if {$COMMAND=="RUN"} {
		after 2000 set cmdvar $cmdnum 
	} else {
		after 100 set cmdvar $cmdnum 
	}
	vwait cmdvar
	puts "[clock seconds]: $cmdnum <-$DEST OK"
	incr cmdnum 
}

proc bkgr {} {
	global OBJM
	if {[expr [clock seconds]]<$OBJM(TNEXTOBJ)} {
		after 2000 {puts "[after info]"; bkgr }
	}
}

set CENT(TOL) 3; set CENT(DRA) 5; set CENT(DDEC) -2; set CENT(RATE) 100
set CENT(WORKZONE) 10; set CENT(TIME) 15
# Object selection:
set OBJM(OBJECT) 1234; set OBJM(RA) "12 34 56"; set OBJM(DEC) "-01 23 45";
set OBJM(TVIS) 20; set OBJM(TWAIT) 600
# Detectors parameters:
set MASS(BKGRRATE) 300

# Centering/guiding starting and background execution procedure.
# Being start with "on", returns when star is centered (return value "CENTERED")
# or star not found (returns "NOSTAR") or centering failed (corrector physically
# disconnected or whatsoever, returns "OFFSET"). After successful centering
# calls itself regularly, untill stopped by a call with "off" parameter. 
proc centering {{turn guide}} {
	global CENT
	if {$turn=="ON"} {
		set CENT(STATE) "ON"
		# being turned ON, procedure must center within CENT(TIME) seconds:
		set CENT(AFTER) [after [expr $CENT(TIME)*1000] {set $CENT(STATE) "OFF"}]
	} elseif {$turn=="OFF"} {
		set CENT(STATE) "OFF"
	}

	while 1 {
	# Trace the state is not set OFF:
	if {$CENT(STATE)=="OFF"} {
		# Unset the normal status (OFFSET or CENTERED) before exit:
		set $CENT(STATUS) "UNKNOWN"; 
		return $CENT(STATUS)
	}

	# Measure the position of the star in the frame and return
	# STATUS=READY|NOSTAR|ER??? and, if READY, DRA=dra DDEC=ddec in addition
	cmd CENT RUN
	if {$CENT(STATUS)=="NOSTAR"} {
		set $CENT(STATE) "OFF"
		return $CENT(STATUS)
	}
	# check the star is within the tolerance circle:
	set dist [expr sqrt($CENT(DRA)*$CENT(DRA)+$CENT(DDEC)*$CENT(DDEC))]
	if {$dist>$CENT(TOL)} {
		# Apply the correction. Return STATUS=READY|ER???
		cmd CORR RUN DRA=[expr -$CENT(DRA)] DDEC=[expr -$CENT(DDEC)]
	}
	# extend status to the quality of centering measure:
	if {$dist>$CENT(WORKZONE)} {
		set $CENT(STATUS) "OFFSET"
	} else {
		set $CENT(STATUS) "CENTERED"
	}
	# wait a bit before next iteration:
	after $CENT(RATE) centering
}

# next command is an infinite loop: 
#while 1 {

# select current object: return STATUS=READY|ER??? OBJECT=nnnn RA="hh mm ss"
# DEC="sdd mm ss" TVIS=timebest
cmd	OBJM	GET OBJECT RA DEC TVIS
# next object selection time:
set OBJM(TNEXTOBJ) [expr $OBJM(TVIS)+[clock seconds]]	

# Pointing:
# point telescope to the selected direction and return STATUS=READY|ER???
cmd	TLSP	RUN RA=$OBJM(RA) DEC=$OBJM(DEC)

# Start Centering procedure:
switch [centering on] {
	"OFFSET"
		-
	"NOSTAR"
		{ 
		# Clouds? "OFFSET" might be caused by wind; wait until better weather
			after [expr $OBJM(TWAIT)*1000] continue 
		}
}			
} else {
	# inform detectors about the object set: return STATUS=READY|ER???
	cmd	DIMM SET OBJECT=$OBJM(OBJECT)
	cmd	MASS SET OBJECT=$OBJM(OBJECT)
	# Time to measure bkgr:
	set $MASS(BKGRT) 0
	# Start the measurement loop for the time-visible period:
	while {[clock seconds]<$OBJM(TNEXTOBJ)} {
		if {[clock seconds]>$MASS(BKGRT)} {
			# Time to measure backround:
			
			set MASS(BKGRT) [expr [clock seconds] + $MASS(BKGRRATE)*1000]
		}
		# Get status of the first detector:
		cmd DIMM GET STATUS
		if {$DIMM(STATUS)=="READY"} 
			{after idle {cmd DIMM RUN}}
		cmd MASS GET STATUS
		if {$MASS(STATUS)=="READY"} 
			{after idle {cmd MASS RUN}}
		after $DIMM
	}
}
#  Stop centering: End of object measurements
set CENT(STATE) "OFF"
#}
