# this Tcl source contains the procedures for converting the message into
# list of command identifier, keyword, param1, value1,... - parse_command,
# and for back creation of the message - compose_command

proc parse_command scmd {
	# In all instances of parameter values enclosed by double-quotes, remove
	# these quotes and replace the spaces between them with double underscores. Before,
	# replace the escaped doublequotes with DOUBLEQUOTE in advance. This all is
	# needed to allow the correct splitting of the answer in a list.
	set scmd [string map { \\\" DOUBLEQUOTE } $scmd]
	while {[set q1p [string first \" $scmd]]>=0} {
		set q2p [string first \" $scmd [expr $q1p+1]]
		# In case closing quote is forgotten - assume it in the end of line.
		if {$q2p<0} {set q2p [string length $scmd]}
		set scmd [string replace $scmd $q1p $q2p [string map \
			{ \" {} \  __ } [string range $scmd $q1p $q2p]]]
	}
	# split string command into list and remove empty entries 
	# (caused by doubled spaces):
	set rl [split $scmd]
	while {[set i [lsearch -exact $rl {}]]>=0} {
		set rl [lreplace $rl $i $i]
	}

	# Initialize the resulting list with comid and keyword.
	# Make the keyword uppercase and append it:
	lappend resl [lrange $rl 0 0] [string toupper [lindex $rl 1]]

	set rl [lreplace $rl 0 1]
	# Parse rest of scmd and assign the replied quantities to the component 
	# parameters:
	foreach pv $rl {
		set esp [string first = $pv]
		# Avoid non-conventional replies with non "PARAM=VALUE" additions:
		if {$esp<0} {
			set param [string tolower $pv]
			set value "switch"
		} else {
			set param [string tolower [string range $pv 0 [expr $esp-1]]]
			set value [string range $pv [expr $esp+1] end]
			# put back the spaces and doublequotes mapped above:
			set value [string map { DOUBLEQUOTE \\\" __ \  } $value]
		}
		lappend resl $param $value
	}
	return $resl
}

proc compose_command lcmd {
	set scmd "[lindex $lcmd 0] [lindex $lcmd 1]"
	set i 2
	set l [llength $lcmd]
	while {$i<$l} {
		set param [lindex $lcmd $i]
		set value [lindex $lcmd [expr $i+1]]
		if {$value=="switch"} {
			set scmd "$scmd $param"
		} else {
			if {[string first \  $value]>=0} {
				set value \"$value\"
			}
			set scmd "$scmd $param=$value"
		}
		incr i
		incr i
	}
	return $scmd
}